pub struct MclmcStats<P: HasDims, H: Storable<P>, A: Storable<P>, Pt: Storable<P>> {
pub chain: u64,
pub draw: u64,
pub num_steps: u64,
pub energy_change: f64,
pub log_weight: f64,
pub tuning: bool,
pub hamiltonian: H,
pub adapt: A,
pub point: Pt,
pub average_step_size: f64,
pub divergence: DivergenceStats,
/* private fields */
}Expand description
Per-draw sampler statistics for MclmcChain.
Fields
chain: u64draw: u64num_steps: u64energy_change: f64log_weight: f64Unnormalized log importance weight for this draw:
log_weight = log(step_size_factor) − energy_change.
step_size_factor is the factor applied to the base step size for the
final leapfrog step (1.0 when no dynamic retry occurred).
log(factor) corrects for varying sampling density under dynamic step
size reduction; it is 0.0 (no correction) when the step is taken at
full size. The −energy_change term corrects for integration error.
See Robnik & Seljak (2023), arXiv:2212.08549.
tuning: boolhamiltonian: Hadapt: Apoint: Ptaverage_step_size: f64divergence: DivergenceStatsTrait Implementations
Sourceimpl<P: Debug + HasDims, H: Debug + Storable<P>, A: Debug + Storable<P>, Pt: Debug + Storable<P>> Debug for MclmcStats<P, H, A, Pt>
impl<P: Debug + HasDims, H: Debug + Storable<P>, A: Debug + Storable<P>, Pt: Debug + Storable<P>> Debug for MclmcStats<P, H, A, Pt>
Sourceimpl<P: HasDims, H: Storable<P>, A: Storable<P>, Pt: Storable<P>> Storable<P> for MclmcStats<P, H, A, Pt>
impl<P: HasDims, H: Storable<P>, A: Storable<P>, Pt: Storable<P>> Storable<P> for MclmcStats<P, H, A, Pt>
fn names(parent: &P) -> Vec<&str>
fn item_type(parent: &P, item: &str) -> ItemType
fn dims<'a>(parent: &'a P, item: &str) -> Vec<&'a str>
Sourcefn event_dim(parent: &P, item: &str) -> Option<&'static str>
fn event_dim(parent: &P, item: &str) -> Option<&'static str>
Return the name of the primary dimension for the given field, or
None
if the field uses the default draw dimension.fn get_all<'a>(&'a mut self, parent: &'a P) -> Vec<(&'a str, Option<Value>)>