LogpError

Trait LogpError 

pub trait LogpError: Error + Send {
    // Required method
    fn is_recoverable(&self) -> bool;
}
Errors that happen when we evaluate the logp and gradient function

Required Methods

fn is_recoverable(&self) -> bool

Unrecoverable errors during logp computation stop sampling, recoverable errors are seen as divergences.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors